PrefabFiles = {
	"womp",
	"womp_none",
	"droplets",
	"puddle_move_fx",
	"tentaclespots_placer",
}

Assets = {
    Asset( "IMAGE", "images/saveslot_portraits/womp.tex" ),
    Asset( "ATLAS", "images/saveslot_portraits/womp.xml" ),

    Asset( "IMAGE", "images/selectscreen_portraits/womp.tex" ),
    Asset( "ATLAS", "images/selectscreen_portraits/womp.xml" ),
	
    Asset( "IMAGE", "images/selectscreen_portraits/womp_silho.tex" ),
    Asset( "ATLAS", "images/selectscreen_portraits/womp_silho.xml" ),

    Asset( "IMAGE", "bigportraits/womp.tex" ),
    Asset( "ATLAS", "bigportraits/womp.xml" ),
	
	Asset( "IMAGE", "images/map_icons/womp.tex" ),
	Asset( "ATLAS", "images/map_icons/womp.xml" ),
	
	Asset( "IMAGE", "images/avatars/avatar_womp.tex" ),
    Asset( "ATLAS", "images/avatars/avatar_womp.xml" ),
	
	Asset( "IMAGE", "images/avatars/avatar_ghost_womp.tex" ),
    Asset( "ATLAS", "images/avatars/avatar_ghost_womp.xml" ),
	
	Asset( "IMAGE", "images/avatars/self_inspect_womp.tex" ),
    Asset( "ATLAS", "images/avatars/self_inspect_womp.xml" ),
	
	Asset( "IMAGE", "images/names_womp.tex" ),
    Asset( "ATLAS", "images/names_womp.xml" ),
	
    Asset( "IMAGE", "bigportraits/womp_none.tex" ),
    Asset( "ATLAS", "bigportraits/womp_none.xml" ),
	
	Asset("SOUNDPACKAGE", "sound/womp.fev"),
	Asset("SOUND", "sound/womp.fsb"),
	
}

RemapSoundEvent( "dontstarve/characters/womp/death_voice", "womp/characters/womp/death_voice" )
RemapSoundEvent( "dontstarve/characters/womp/hurt", "womp/characters/womp/hurt" )
RemapSoundEvent( "dontstarve/characters/womp/talk_LP", "womp/characters/womp/talk_LP" )

local require = GLOBAL.require
local STRINGS = GLOBAL.STRINGS

-- The character select screen lines
STRINGS.CHARACTER_TITLES.womp = "The Abyssal"
STRINGS.CHARACTER_NAMES.womp = "Womp"
STRINGS.CHARACTER_DESCRIPTIONS.womp = "*Benefits from moisture, suffers when dry\n*Has very good grip\n*Is a monster"
STRINGS.CHARACTER_QUOTES.womp = "\"Quit stomping around up there!\""

-- Custom speech strings
STRINGS.CHARACTERS.WOMP = require "speech_womp"

-- The character's name as appears in-game 
STRINGS.NAMES.WOMP = "Womp"

-- The default responses of examining the character
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WOMP = 
{
    GENERIC = "It's Womp!",
    ATTACKER = "That Womp looks shifty...",
    MURDERER = "Murderer!",
    REVIVER = "Womp, friend of ghosts.",
    GHOST = "Womp could use a heart.",
}

AddMinimapAtlas("images/map_icons/womp.xml")

-- Add mod character to mod character list. Also specify a gender. Possible genders are MALE, FEMALE, ROBOT, NEUTRAL, and PLURAL.
AddModCharacter("womp", "MALE")

-- Plantable tentacles.
local function GiveSpotsBack(pt) -- Something removes the item outside of the ondeploy method for some reason, so I gotta give the player spots back if their plant fails. Lame and hacky.
	local spots = GLOBAL.SpawnPrefab("tentaclespots")
	spots.Transform:SetPosition(pt:Get())
	spots.components.inventoryitem:OnDropped()
end
local function PlantableSpots(inst, pt, deployer)
	if deployer:HasTag("swampbro") then
		if deployer.components.moisture:GetMoisture() > 24 then
			local tenta = GLOBAL.SpawnPrefab("tentacle")
			tenta.Transform:SetPosition(pt:Get())
			tenta.sg:GoToState("attack_post")
			
			deployer.components.moisture:DoDelta(-25)
			deployer.SoundEmitter:PlaySound("dontstarve_DLC001/common/HUD_wet_close")
			inst:Remove()
		else
			deployer.components.talker:Say("Womp is too dry for that!")
			GiveSpotsBack(pt)
		end
	elseif deployer.components.talker ~= nil then
		deployer.components.talker:Say(GLOBAL.GetString(deployer, "ACTIONFAIL_GENERIC"))
		GiveSpotsBack(pt)
	end
end
AddPrefabPostInit("tentaclespots", function(inst)
	if GLOBAL.TheWorld.ismastersim then
		inst:AddComponent("deployable")
		inst.components.deployable:SetDeployMode(GLOBAL.DEPLOYMODE.PLANT)
		inst.components.deployable.ondeploy = PlantableSpots
	end
end)

-- Override tentacle's targeting so they won't target us unless provoked
local function retargetfn_Reg(inst) 
    return GLOBAL.FindEntity(
        inst,
        TUNING.TENTACLE_ATTACK_DIST,
        function(guy) 
            return guy.prefab ~= inst.prefab
                and guy.entity:IsVisible()
                and not guy.components.health:IsDead()
                and (guy.components.combat.target == inst or
                    guy:HasTag("character") or
                    guy:HasTag("monster") or
                    guy:HasTag("animal"))
        end,
        { "_combat", "_health" },
        { "prey", "swampbro" })
end
-- Remove sanity aura when near tentacles.
local function CalcSanityAura(inst, observer)
    return observer:HasTag("swampbro") and 0 or -TUNING.SANITYAURA_MED
end
AddPrefabPostInit("tentacle", function(inst)
    if GLOBAL.TheWorld.ismastersim then
        inst.components.combat:SetRetargetFunction(GLOBAL.GetRandomWithVariance(2, 0.5), retargetfn_Reg)
		inst.components.sanityaura.aurafn = CalcSanityAura
    end
	
	inst:AddTag("tentacle")
end)
-- Override tenatcle arm's targeting so they won't target us... ever, apparently.
local function retargetfn_Arm(inst)
    return GLOBAL.FindEntity(inst, TUNING.TENTACLE_PILLAR_ARM_ATTACK_DIST, function(guy) 
        if not guy.components.health:IsDead() then
            return (not (guy.prefab == inst.prefab))
        end
    end,
    {"_combat","_health"},-- see entityscript.lua
    {"prey", "swampbro"},
    {"character","monster","animal"}
    )
end
AddPrefabPostInit("tentacle_pillar_arm", function(inst)
    if GLOBAL.TheWorld.ismastersim then
        inst.components.combat:SetRetargetFunction(GLOBAL.GetRandomWithVariance(1, 0.5), retargetfn_Arm)
		inst.components.sanityaura.aurafn = CalcSanityAura
    end
	
	inst:AddTag("tentacle")
end)
-- Adding a positive sanity aura to tentacle pillars, cause they're cool.
local function PillarAura(inst, observer) 
	return observer:HasTag("swampbro") and TUNING.SANITYAURA_HUGE or 0
end
AddPrefabPostInit("tentacle_pillar", function(inst)
	if GLOBAL.TheWorld.ismastersim then
		inst:AddComponent("sanityaura")
		inst.components.sanityaura.aurafn = PillarAura
	end
	
	inst:AddTag("tentacle")
end)

--Preventing Bearger disarm
AddPrefabPostInit("bearger", function(inst) --Thanks DarkXero
	if not GLOBAL.TheWorld.ismastersim then
		return
	end
	local listeners = inst.event_listeners.onhitother
	local listener_fns = listeners[inst]
	local old = listener_fns[1]
	listener_fns[1] = function(inst, data)
		local other = data.target
		if other and other.prefab == "womp" then
			return
		end
		return old(inst, data)
	end
end)
--Preventing moose/goose disarm
AddStategraphPostInit("moose", function(sg) --Thanks again DarkXero
	local disarm_state = sg.states.disarm
	local old = disarm_state.timeline[5].fn
	disarm_state.timeline[5].fn = function(inst)
		local target = inst.components.combat.target
		if target and target.prefab == "womp" then
			inst.CanDisarm = false
			return	
		end
		old(inst)
	end
end)

-- Override catcoon's targeting so they don't like us.
local function RetargetFn_Cat(inst)
    return GLOBAL.FindEntity(inst, TUNING.CATCOON_TARGET_DIST,
        function(guy)
        	if guy:HasTag("catcoon") then
        		return 	not (inst.components.follower and inst.components.follower:IsLeaderSame(guy))
        				and not (inst.components.follower and guy.components.follower and inst.components.follower.leader == nil and guy.components.follower.leader == nil)
        				and guy.components.health
	            		and not guy.components.health:IsDead()
	            		and inst.components.combat:CanTarget(guy)
        	else
            	return 	((guy:HasTag("monster") or guy:HasTag("smallcreature") or guy:HasTag("swampbro"))
	            		and guy.components.health
	            		and not guy.components.health:IsDead()
	            		and inst.components.combat:CanTarget(guy)
	            		and not (inst.components.follower and inst.components.follower.leader ~= nil and guy:HasTag("abigail")))
            			and not (inst.components.follower and inst.components.follower:IsLeaderSame(guy))
	            	or 	(guy:HasTag("cattoyairborne")
            			and not (inst.components.follower and inst.components.follower:IsLeaderSame(guy)))
	        end
        end)
end
AddPrefabPostInit("catcoon", function(inst)
    if GLOBAL.TheWorld.ismastersim then
        inst.components.combat:SetRetargetFunction(3, RetargetFn_Cat)
    end
end)
-- Override bunnyman's targeting so they don't like us either.
local function RetargetFn_Bun(inst)
    return GLOBAL.FindEntity(inst, TUNING.PIG_TARGET_DIST,
        function(guy)
            return inst.components.combat:CanTarget(guy)
                and ((guy:HasTag("monster") or guy:HasTag("swampbro"))
                    or (guy.components.inventory ~= nil and
                        guy:IsNear(inst, TUNING.BUNNYMAN_SEE_MEAT_DIST) and
                        guy.components.inventory:FindItem(function(item) return item.components.edible ~= nil and item.components.edible.foodtype == GLOBAL.FOODTYPE.MEAT end) ~= nil))
        end,
        { "_combat", "_health" }, -- see entityreplica.lua
        nil,
        { "monster", "player", "swampbro" })
end
AddPrefabPostInit("bunnyman", function(inst)
    if GLOBAL.TheWorld.ismastersim then
        inst.components.combat:SetRetargetFunction(3, RetargetFn_Bun)
    end
end)
--Override pigman's targeting so they don't like us either either.
AddPrefabPostInit( "pigman", function(inst)
    if not GLOBAL.TheWorld.ismastersim then
        return inst
    end

    local _RegRetargetFn = inst.components.combat.targetfn
    local function RetargetFn_Pig(inst)
        local target = GLOBAL.FindEntity(
            inst,
            TUNING.PIG_TARGET_DIST,
            function(guy)
                return (guy.LightWatcher == nil or guy.LightWatcher:IsInLight())
                    and inst.components.combat:CanTarget(guy)
            end,
            { "swampbro" }
		)
        if target == nil then
            target = _RegRetargetFn(inst)
        end

        return target
    end
	-- Also make pigs still continue to not like us after they turn back from werebeasts.
	local _regFn = inst.components.werebeast.onsetnormalfn
	inst.components.werebeast:SetOnNormalFn(function(inst)  
		_regFn(inst)
		inst.components.combat:SetRetargetFunction(3, RetargetFn_Pig)
	end)
	
    inst.components.combat:SetRetargetFunction(3, RetargetFn_Pig)
end)